#ifndef GLOBPREP_H
#define GLOBPREP_H

#include "vector.h"
#include "siftop.h"
#include "alias.h"
#include "global.h"
#include "loadn.h"
#include "dloadn.h"
#include "loadel.h"
#include "bocon.h"
#include "inicon.h"
#include "dbcrs.h"
#include "dbmat.h"
#include "edgeload.h"
#include "surfload.h"
#include "elsurfload.h"
#include "pointset.h"
#include "tempload.h"

extern siftop  *Top;    ///< Sifel topology
extern long     Npropn; ///< number of entries with nodal property
extern long     Nprope; ///< number of entries with element property

// Folowing arrays have size equal to number of lines with property description
//  of nodes(elements).
// This numbers are specified in the input file on the first line before section
//  described nodals or elements property
extern long      *Nnp;    ///< property numbers for nodes
extern long      *Nep;    ///< property numbers for elements
extern long      *Ndofn;  ///< array of number of dof for nodes
extern bocon    **Boc;    ///< array of pointers to the nodal boundary conditions structures
extern long     **Boccn;  /**< array with bocon squence number (0 : if Boc[i] == NULL)
                             (positive number : if Boc[i] != NULL)*/
extern loadel   **Lel;    ///< array of pointers to the element load structures
extern loadn    **Lno;    ///< array of pointers to the nodal load structures
//extern dloadel  **Dlel; // array of pointers to the dynamic element load structures
extern dloadn   **Dlno;   ///< array of pointers to the dynamic nodal load structures
extern elemtype  *Tel;    ///< array of element types
extern strastrestate *Ssse;///< stress strain state for plane element
extern long      *Sel;    ///< array with element shape number
extern long      *Nmel;   ///< number of materials on the element
extern mattype  **Mel;    ///< array of material types
extern long     **Melid;  ///< array of material indeces
extern crsectype *Ncs;    ///< array of cross-section types in the nodes
extern long      *Ncsi;   ///< array of cross-section indeces in the nodes
extern long     **Nccn;   ///< array of pointers to common code numbers in the nodes
extern tempload **Ntemp;  ///< array with temperature load of nodes
extern long       Ntempl; ///< indicator whether temperature load is given in the current problem
extern crsectype *Ecs;    ///< array of cross-section types in the elements
extern long      *Ecsi;   ///< array of cross-section indeces in the elements
extern vector   **Nlsv;   ///< array of vectors of local systems in the nodes
extern vector   **Elsv;   ///< array of vectors of local systems in the elements
extern long       Nels;   ///< number of local coordinate systems in the integration points
extern dbmat     *Dbmat;  ///< database of materials and their parameters
extern dbcrs     *Dbcrs;  ///< database of cross-sections and their parameters
extern long      *Tndofn; ///< array of number of dof for every node
extern elemtype  *Tet;    ///< array of element type for each element
extern long      *Nmu;    ///< array with indicators of used material types from material database
extern long      *Ncu;    ///< array with indicators of used cross-section types from cross-section database
extern long       Nlc;    ///< number of loading cases
extern long       Nslc;   ///< number of loading cases
extern long       Nlay;   ///< number of layers for layered problems
extern long       Nspd;   ///< number of values of static prescribed displacements
extern long       Ndpd;   ///< number of values of dynamic prescribed displacements
extern long       Csrin;  ///< number of renumbered index in function wr_nodes
extern inicon   **Inic;   ///< array of pointers to the nodal initial conditions structures
extern long       Ninic;  ///< indicator whether initial conditions are given in the current problem
extern long      *Enip;   ///< array with number of integration point for each element
extern edgeload **Edgl;   ///< array with edge load
extern surfload **Surfl;  ///< array with surface load
extern elsurfload  **Elsurfl; ///< array with element surface load
extern elemposition *Eep;     ///< array with position indicator of point where stresses and strains will be evaluated
extern pointset    **Eps;     ///< array with position point sets
extern gfunct       *Tf;      ///< array of time functions used in time dependent mechanical problems for each load case
extern dynload       Tdynload;///< type of dynamic load
extern long         *Tfel;    ///< array of indeces of time functions for elements in growing structure problems
#endif
