#ifndef GRAPHMATTIME_H
#define GRAPHMATTIME_H

#include "iotools.h"
#include "alias.h"
struct matrix;
class Equation;
class tablefunct;



/**
  This class defines special type of material model which is
  used for the spring support with nonlinear stiffness. The
  constitutive equation is described with force/displacement
  diagram. The diagram can be given by the table or by the
  expression written to the string which is parsed. The several
  equations for the different ranges can be given.

  In case that diagram is given by table with Dirac interpolation, 
  the y-values in the table represents directly stiffnesses instead of
  forces.
*/
class graphmattime
{
  public :
   graphmattime(void);
   ~graphmattime(void);
   long read (XFILE *in);
   void matstiff (matrix &d,long ipp);
   void nlstresses (long ipp, long im, long ido);

   /// type of graph
   graphtype gt;
   /// number of function
   long numf;
   /// stiffness for linear elastic material
   double k;
   /// array of functions of graph (displacement-force graph)
   Equation **eq;
   /// array of functions of graph (displacement-stiffness graph)
   Equation **deq;
   /// array of limit values for each function
   double   *limval;
   /// table of values of graph
   tablefunct *tab;
};

#endif
