#include "lssolver.h"
#include "global.h"
#include "globmat.h"
#include "loadcase.h"
#include "gmatrix.h"
#include "mechprint.h"
#include "node.h"
#include "elemswitch.h"
#include <string.h>

/**
   function solves linear statics problems
   
   function contains the following steps:
   1. stiffness matrix assembling
   2. right hand side assembling (load vectors assembling)
   3. solution of equation systems by choosen method
   4. strain computation (eligible)
   5. stress computation (eligible)
   6. reaction computation (eligible)
   7. results printing
   
   JK
*/
void solve_linear_statics ()
{
  long i,n;
  double *lhs,*rhs,*fl,*fi;
  
  //  stiffness matrix assembling
  stiffness_matrix (0);
  
  //  array is allocated in Lsrs
  lhs = Lsrs->give_lhs (0);
  rhs = Lsrs->give_rhs (0);
  
  //  number of mechanical degrees of freedom
  n=Ndofm;
  //  vector of prescribed force loads, it does not contain forces caused by temperature, etc.
  fl   = new double [n];
  nullv (fl,n);
  //  vector of resulting internal forces
  fi   = new double [n];
  nullv (fi,n);

  //  loop over load cases
  for (i=0;i<Lsrs->nlc;i++){
    //  right hand side of system
    mefel_right_hand_side (i,rhs);
  }
  
  //Smat->printmat (Out);

  /*
  FILE *out;
  out = fopen ("vektor.txt","w");
  for (i=0;i<Ndofm;i++){
    fprintf (out,"%20.15le\n",rhs[i]);
  }
  fclose (out); 
  Smat->printmat (Out);
  
  out = fopen ("dof.txt","w");
  for (i=0;i<Gtm->nn;i++){
    fprintf (out,"%6ld  %6ld %6ld %6ld\n",i,Gtm->gnodes[i].cn[0],Gtm->gnodes[i].cn[1],Gtm->gnodes[i].cn[2]);
  }
  fclose (out); 

  out = fopen ("sit.txt","w");
  fprintf (out,"%6ld\n",Gtm->nn);
  for (i=0;i<Gtm->nn;i++){
    fprintf (out,"%6ld  %le %le %le\n",i,Gtm->gnodes[i].x,Gtm->gnodes[i].y,Gtm->gnodes[i].z);
  }
  fprintf (out,"%6ld\n",Gtm->ne);
  for (i=0;i<Gtm->ne;i++){
    fprintf (out,"%6ld   %6ld %6ld %6ld %6ld\n",i,Gtm->gelements[i].nodes[0],Gtm->gelements[i].nodes[1],Gtm->gelements[i].nodes[2],Gtm->gelements[i].nodes[3]);
  }
  fclose (out); 
  */

  /*
  FILE *out;
  out = fopen ("matice_pred.m","w");
  Smat->printmat (out);
  fclose (out); 
  */
  
  //  solution of equation system
  for (i=0;i<Lsrs->nlc;i++){
    Mp->ssle->solve_system (Gtm,Smat,Lsrs->give_lhs(i),Lsrs->give_rhs(i),Out);
  }
  
  /*
  out = fopen ("matice_po.m","w");
  Smat->printmat (out);
  fclose (out); 
  */
  

  print_init(-1, "wt");    
  for (i=0;i<Lsrs->nlc;i++){
    //  computes required quantities
    compute_req_val (i);
    //  prints required quantities    
    //mefel_right_hand_side (i,rhs); // rhs was rewritten by the solver call -> rebuild load vector
    //print_step(i, 0, 0.0, Lsrs->give_rhs(i));
    mefel_right_hand_side (i,Lsrs->give_rhs(i),fl);//tady debug??!!
    print_step(i, 0, 0.0, fl);//tady debug??!!
    internal_forces(i, fi);
    print_step(i, 1, 1.0, fi);//tady debug??!!
  }
  print_close();

  
  if (Mp->adaptivity)
    Ada->run (2,0,0); // 2 - for testing

  delete [] fl;  
}

