#ifndef MTSOLVER_H
#define MTSOLVER_H

#include <stdio.h>
#include "selection.h"
#include "timecontr.h"


/// function invokes solver of time dependent problems
void solve_time_dep_prob ();

/// solver of time dependent problems, viscous models can be used
void visco_solver (long lcid);

/// function performs one time step of visco_solver
void one_step (long lcid, long n, double *f, double *fi, double *fp, double *fb, double *dr, double *r);

/// candidate for removal
void aux_nonlintime_print (FILE *aux, double *r, double l);

// FUNCTIONS OF HARDDISK BACKUP

/// function saves actual step of the solver and internal data for the future restart
void mtsolver_save (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);
/// function saves actual step of the solver and internal data for the future restart in a text format
void mtsolver_save_text (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);
/// function saves actual step of the solver and internal data for the future restart in a binary format
void mtsolver_save_binary (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);
/// function saves actual step of the solver and internal data for the future restart in a text file (single file)
void mtsolver_save_text_single (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);
/// function saves actual step of the solver and internal data for the future restart in a text file (multiple files)
void mtsolver_save_text_multiple (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);
/// function saves actual step of the solver and internal data for the future restart in a binary file (single file)
void mtsolver_save_binary_single (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);
/// function saves actual step of the solver and internal data for the future restart in a binary file (multiple files)
void mtsolver_save_binary_multiple (double *r, double *fp, long ni, double time, double dt, timecontr &tc, long n);

/// function restores saved step of the solver and internal data and restarts computation
void mtsolver_restore (double *r, double *fp, long &ni, double &time, double &dt, timecontr &tc, long &n);
/// function restores saved actual step of the solver and internal data and restarts computation (text format)
void mtsolver_restore_text (double *r, double *fp, long &ni, double &time, double &dt, timecontr &tc, long &n);
/// function restores saved actual step of the solver and internal data and restarts computation (binary format)
void mtsolver_restore_binary (double *r, double *fp, long &ni, double &time, double &dt, timecontr &tc, long &n);
/// function restores saved actual step of the solver and internal data and restarts computation from a text file
void mtsolver_restore_text_single (double *r, double *fp, long &ni, double &time, double &dt, timecontr &tc, long &n);
/// function restores saved actual step of the solver and internal data and restarts computation from text files
void mtsolver_restore_text_multiple (double *r, double *fp, long &ni, double &time, double &dt, timecontr &tc, long &n);
/// function restores saved actual step of the solver and internal data and restarts computation from a binary file
void mtsolver_restore_binary_single (double *r,double *fp,long &ni,double &time, double &dt, timecontr &tc, long &n);
/// function restores saved actual step of the solver and internal data and restarts computation from binary files
void mtsolver_restore_binary_multiple (double *r,double *fp,long &ni,double &time, double &dt, timecontr &tc, long &n);


#endif
