#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "nssolver.h"
#include "slopesol.h"
#include "global.h"
#include "globmat.h"
#include "mechprint.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "loadcase.h"
#include "dloadcase.h"
#include "intpoints.h"
#include "auxdatout.h"
#include "mechprint.h"
#include "j2flow.h" // pak vyhodit
#include "chen.h" // pak vyhodit
#include "hardsoft.h" // pak vyhodit
#include "mathem.h"
#include "vector.h"
#include "mtsolver.h"

#include "adaptivity.h"
#include "meshtransfer.h"

/**
   function solves non-linear statics problems
   
   JK
*/
void solve_nonlinear_statics ()
{
  long i;
  double *rhs;

  Mp->lambda = 1.0;  
  rhs=Lsrs->give_rhs (0);
  
  for (i=0;i<Lsrs->nlc;i++){
    
    //  vectors of loading
    //Mb->lc[i*2+0].assemble (0,rhs+(i*2+0)*Ndofm);
    //Mb->lc[i*2+1].assemble (0,rhs+(i*2+1)*Ndofm);
    mefel_right_hand_side (i,rhs);
    
    if (!Mp->adaptivity)
      nonlinear_solver (i);
    else
      nonlinear_solver_adaptiv (i);

 }
  
}

void print_output(long lcid)
{
/*
  //  strains
  if (Mp->straincomp==1){
    Mm->computestrains (lcid);
    Mm->stra.transformvalues (1);
  }
    
  //  stresses
  if (Mp->stresscomp==1){
    Mm->computenlstresses (lcid);
    Mm->stre.transformvalues(0);
  }
  
  //  reactions
  if (Mp->reactcomp==1)
    Mb->lc[lcid].compute_reactions (lcid);
  
    
  //  output data
  Lsrs->output (Out,lcid);    
  // other at the integration points  
  if ((Mp->straincomp==1) || (Mp->stresscomp==1))
    print_other (Out,lcid);
*/
}

void nonlinear_solver (long lcid)
{
  switch (Mp->nlman.tnlinsol){
  case arcl:{
    arclength (lcid,0);
    break;
  }
  case newton:{
    newton_raphson (lcid);
    break;
  }
  case arclrv1:{
    arclengthrv1 (lcid,1.1,1.0e-3);
    break;
  }
  case newtonrv1:{
    newton_raphsonrv1 (lcid,Mp->nlman.rvlam,1.0e-3);
    break;
  }
  case newtonrestart:{
    newton_raphson_restart (lcid);
    break;
  }
  case displctrl:{
    displ_control (lcid);
    break;
  }
  case adaptram:{
    arclengthadapt (lcid,0);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown solver of nonlinear equation system is required");
    fprintf (stderr,"\n in function nonlinear_solver (%s, line %d).\n",__FILE__,__LINE__);
  }
  }

}

void refresh_plast ()
{
  for (long i=0;i<Mm->tnip;i++)
    if ( Mm->j2f[Mm->ip[i].idm[0]].give_consparam (i,0) ){
      Mm->plast = 1;
      return;
    }
}

/**
   if adaptcontrol =
   1 - arclength nyni bezi poprve
   2 - arclength nyni nebezi poprve
   
   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void nonlinear_solver_adaptiv (long lcid)
{
  int width;
  long i,adaptcontrol;
  char file[255];
  
  switch (Mp->nlman.tnlinsol){
  case arcl:{
    if (Mespr==1){
      fprintf (stdout,"\n\n ----------------------------");
      fprintf (stdout,  "\n ***  Arc-length - start  ***\n");
    }
    
    width = 1 + (long)log10((double)Mp->nlman.nial);
    adaptcontrol = 1;
    
    for (i=0; i < Mp->nlman.nial ;){
      // i = number of performed steps
      i = arclength (lcid,adaptcontrol);
      
      if ( i<Mp->nlman.nial || Mp->nlman.nial==1 && (Mp->adaptflag & 16) ){
	if (Mespr==1)  fprintf (stdout,"\n\n NEW MESH will be generated\n");
	
	newmeshgen (width,i);
	sprintf (file,"%s%s.in.%0*ld",Mp->path,Mp->filename,width,i);
	newmeshread (file,lcid);
	
	if ( i<Mp->nlman.nial ){
	  if (Mm->plast == 0)
	    refresh_plast ();
	  
	  arclength15  (lcid);
	}
	else
	  Ada->run (4,width,i); // only for testing
      }
      
      //if (!(Mp->adaptflag & 16)){  sprintf (file,"%s%s.dx.%ld.1",Mp->path,Mp->filename,i);  print_default_2_dx (Gtm,Mp,Mt,0,file);  }
      
      adaptcontrol++;
    }
    
    if (Mespr==1){
      fprintf (stdout,"\n\n ***   Arc-length is finished  ***");
      fprintf (stdout,  "\n *********************************");
    }
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown solver of nonlinear equation system is required");
    fprintf (stderr,"\n in function nonlinear_solver_adaptiv (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}

/**
   function solves system of nonlinear algebraic equation
   by arc-length method (continuation method)
   only one right hand side vector is supported with respect
   to nonlinearity and absence of superposition

   d stands for delta
   dd stands for capital delta
   
   fc - nonproportional %vector
   fp - proportional %vector
   n - order of the system
   
   @param lcid - load case id
   
   25.7.2001
*/
long arclength (long lcid,long adaptcontrol)
{
  long i,j,k,n,ni,ini,stop,modif,li,newmesh, numr;
  double a0,a1,a2,l1,l2,dl,dlmax,dlmin,psi,lambda,blambda,dlambda,ddlambda,norf,norfp,norv,norfa,zero,ierr;
  double lambdao,ss1,ss2,ss3,ss4,ss5;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  
  
  
  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.nial;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilal;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  ierr = Mp->nlman.erral;
  //  length of arc
  dl = Mp->nlman.dlal;
  //  maximum length of arc
  dlmax = Mp->nlman.dlmaxal;
  //  minimum length of arc
  dlmin = Mp->nlman.dlminal;
  //  displacement-loading driving switch
  psi = Mp->nlman.psial;
  
  dlambda=0.0;
  
  //  allocation of auxiliary arrays
  r = new double [n];
  ddr = new double [n];
  u = new double [n];
  v = new double [n];
  f = new double [n];
  fa = new double [n];
  fi = new double [n];
  
  //  pokus
  //double *zaloha,*zaloha2;
  //zaloha = new double [n];
  //zaloha2 = new double [n];
  //for (i=0;i<n;i++){
  //zaloha[i]=0.0;
  //zaloha2[i]=0.0;
  //}
  // konec pokusu
  
  //  initialization phase
  ra = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  
  if (Mp->nlman.hdbackupal == 2){            // termit
    arclopen (li,n,lambda,dl,ra,fp);
  }
  else if (adaptcontrol >= 2){
    arclopen_adapt (li,lambda,blambda,dl);
  }
  else{
    lambda=0.0;
    lambdao=0.0;
    li=0;
  }
  
  //refresh_plast ();  // termit
  
  //  Mp->ns;

  //  norm of proportionality vector
  norfp = ss(fp,fp,n);
  modif=0;


  // assembling reached load vector
  for (j=0;j<n;j++){
    fa[j]=fc[j]+(lambda+dlambda)*fp[j];
  }
  if (li)
    print_init(-1, "at");
  else
  {
    print_init(-1, "wt");
    print_step(lcid, li, lambda, fa);
  }
  
  //fprintf (Out,"\n\n osel   epspuc %lf   sc %lf\n",Mm->chplast[0].hs.epspuc,Mm->chplast[0].hs.sc);
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=li;i<ni;i++){

    fprintf (Out,"\n\n arc-length  prirustek %ld",i);

    //if (i>250)
    //Mp->phase=1;

    stop=1; // termit
    //  backup of left hand side vector
    copyv (ra, r, n);
    //  backup of reached lambda parameter
    blambda=lambda;

    //fprintf (stdout,"\n\n arc-length: increment %ld   lambda %e  dl %e",i,lambda,dl);
/*    if (Mespr==1){
      fprintf (Out,"\n\n *******************************************************************");
      fprintf (Out,"\n arc-length: increment %ld,   lambda=%e   dl=%e",i,lambda,dl);
    }*/
    
    //  assembling of tangent stiffness matrix
    if ((Mp->nlman.stmat==tangent_stiff) || (i == li)){
      stiffness_matrix (lcid);
      Smat->printmat (Out);
    }
    
    //  backup of the fp, in ldl_sky the right hand side will be destroyed
    copyv(fp, f, n);

    //  solution of K(r).v=F
    Smat->solve_system (v,f);

    //  generalized norm of displacement increments
    norv = displincr (v,n);

    //  compute new dlambda increment
    dlambda = dl/sqrt(norv+psi*psi*norfp);
    
    //if (norv+psi*psi*norfp < 1.0e-8)  dlambda=0.0;
    //else dlambda = dl/sqrt(norv+psi*psi*norfp);
    
    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      ra[j]+=ddr[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    subv(fa, fi, f, n);    
    norf=sizev(f,n);
    norfa = sizev(fa, n);
    //if (norfa<1.0e-8){}
    //else norf /= norfa;
    norf /= norfa;

    
    //if (Mespr==1)  fprintf (stdout,"\n ddlambda %e    dlambda %e   norf %e  ierr %e",ddlambda,dlambda,norf,ierr);
    if (Mespr==1)  fprintf (stdout,"\n increment %ld     norv %e      norf %e",i,norv,norf);
    
    if (norf<ierr){
      // ******************************************
      //  no inner iteration loop is required  ***
      // ******************************************
      modif++;

      if (modif>1){
	//  arc length modification
	dl*=2.0;
	if (dl>dlmax)  
          dl=dlmax;
	modif=0;
	if (Mespr==1){
	  fprintf (stdout,"\n arc-length must be modified (dl increase) dl=%e",dl);
//	  fprintf (Out,"\n arc-length must be modified (dl increase) dl=%e",dl);
	}
      }
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, fa);
      print_flush();
    }
    else{
      // ****************************
      //  inner iteration loop  ****
      // ****************************
      stop=0;
      for (j=0;j<ini;j++){
	if (Mp->stmat==tangent_stiff)
	  stiffness_matrix (lcid);

	fprintf (Out,"\n arc-length  vnitrni smycka  %ld %ld",i,j);
	
	//  back substitution
	Smat->solve_system (u,f);
	
        copyv(ddr, f, n);
        addv(ddr, u, n);
	//  coefficient of quadratic equation
	quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
	/*
	  fprintf (Out,"\n\n\n Kontrola kvadraticke rovnice");
	  fprintf (Out,"\n norfp     %15.10le",norfp);
	  fprintf (Out,"\n norv      %15.10le",norv);
	  fprintf (Out,"\n (ddr,v)   %15.10le",ss(ddr,v,n));
	  fprintf (Out,"\n (ddr,ddr) %15.10le",ss(ddr,ddr,n));
	  fprintf (Out,"\n dl        %15.10le",dl);
	  fprintf (Out,"\n ddlambda  %15.10le",ddlambda);
	  fprintf (Out,"\n psi       %15.10le",psi);
	  fprintf (Out,"\n a2        %15.10le",a2);
	  fprintf (Out,"\n a1        %15.10le",a1);
	  fprintf (Out,"\n a0        %15.10le",a0);
	  fprintf (Out,"\n discrim   %15.10le",a1*a1-4.0*a2*a0);
	  fprintf (Out,"\n\n");
	*/
	
	//  solution of quadratic equation
        numr = solv_polynom_2(a2, a1, a0, l1, l2);
        switch (numr)
	{
	  case -1:
            fprintf (stderr,"\n\n infinite number of solution of constrained condition in function arclength");
            break;
	  case 0:
            fprintf (stderr,"\n\n nonsolvable constrained condition in function arclength");
            break;
    	  case 1:
            dlambda = l1;
            break;
	  default:
            break;
	}
	ss1=0.0;  ss2=0.0;
	ss3=0.0;  ss4=0.0;  ss5=0.0;
	for (k=0;k<n;k++){
	  ss1+=(ddr[k]+l1*v[k])*f[k];
	  ss2+=(ddr[k]+l2*v[k])*f[k];
	  ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
	  ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
	  ss5+=f[k]*f[k];
	}
	
	/*
	if (ss1/ss3/ss5>ss2/ss4/ss5)  
          dlambda=l1;
	else                          
          dlambda=l2;
	*/


	if (fabs(l1)>fabs(l2))  dlambda=l2;
	else dlambda=l1;

	
	/*
	k=n-1;
	if (ra[k]+u[k]+l1*v[k] > ra[k]+u[k]+l2*v[k])  dlambda=l1;
	else  dlambda=l2;
	*/
	/*
	if (Mp->phase==1){
	  k=n-1;
	  if (ra[k]+u[k]+l1*v[k] > ra[k]+u[k]+l2*v[k])  dlambda=l1;
	  else  dlambda=l2;
	}
	else{
	  if (fabs(l1)>fabs(l2))  dlambda=l2;
	  else dlambda=l1;
	}
	fprintf (Out,"\n\n arc-length   %le  %le     %le %le   %le\n",ra[k]+u[k]+l1*v[k],ra[k]+u[k]+l2*v[k],l1,l2,dlambda);
	*/
	
	//fprintf (stdout,"\n increment  %ld     inner loop  %ld   x1=%e x2=%e   dlambda %e",i,j,l1,l2,dlambda);
	//fprintf (stdout,"\n increment  %ld     inner loop  %ld   x1=%e x2=%e   ddlambda %e",i,j,l1,l2,ddlambda);

	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	//fprintf (stdout,"   ddlambda %e",ddlambda);
	
	//fprintf (Out,"\n ddlambda %e     dlambda %e",ddlambda,dlambda);
	//  computation of internal forces
	internal_forces (lcid,fi);
        subv(fa, fi, f, n);	
	norf=sizev(f,n);
	norfa = sizev(fa, n);
	norf /= norfa;
	//if (norfa<1.0e-8){}
	//else norf /= norfa;
	
	if (Mespr==1){
	  fprintf (stdout,"\n    norf=%e  ierr=%e",norf,ierr);
	  //fprintf (Out,"\n increment  %ld     inner loop  %ld    norf=%e",i,j,norf);
	}

	if (norf<ierr){
	  lambda+=ddlambda;
	  Mm->updateipval ();
	  stop=1; 
	  compute_req_val (lcid);
          print_step(lcid, i+1, lambda, fa);
          print_flush();
	  break;
	}
      }
      modif=0;
      if (stop==0){
	//  modification of the arc length
	dl/=2.0;
	if (dl<dlmin){
          dl=dlmin;  
          break; 
        }
	if (Mespr==1){
	  fprintf (stdout,"\n arc length must be modified (dl decrease) dl=%e",dl);
	  //fprintf (Out,"\n arc length must be modified (dl decrease) dl=%e",dl);
	}
	//  restoring of left hand side vector
        copyv(r, ra, n);
	//  restoring of lambda parameter
	lambda=blambda;
      }
    }
    
    fprintf (stdout,"\n increment %ld    total lambda %e",i,lambda);

    if (stop==0)
      continue;
    
    if (Mp->adaptflag & 16 && i%10 && Mm->plast)
      continue;
    
    newmesh = 0;
    if (Mp->adaptivity && Mp->nlman.nial > (i+1) && !( (Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))){
      
      // print of fxlambda second part is in adaptivity.cpp
      //FILE *ff;
      //sprintf (file,"%sfxlambda.dat",Mp->path);
      //ff = fopen (file,"a");
      //if (ff==NULL) fprintf (stderr,"\n ff file has not been opened.\n\n");
      //fprintf (ff,"  %20.10f",lambda);
      //fclose (ff);
      
      long width = 1 + (long)log10((double)Mp->nlman.nial);
      newmesh = Ada->run (1,width,i);
    }
    
    if (newmesh){
      i++;             // toto se da odstranit
      break;
    }
    // timret
    //print_output(lcid);
  }
  
  // ------------------------------------
  //  finish of main iteration loop  ----
  // ------------------------------------
  
  fprintf (Out,"\n\n blb  %lf",Lsrs->lhs[0]);

  
  if (Mp->nlman.hdbackupal==1)
    arclsave (i,n,lambda,dl,ra,fp);

  // termit
  if (Mp->adaptivity)
    if ((Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))
      Ada->run (0,0,0);  // only for testing
    else
      if (i < ni)
	arclsave_adapt (i,lambda,0,dl);
  
  //if (Mp->adaptivity)   arclsave_adapt (i,blambda,ddlambda,dl);
  // timret
  print_close();
  delete [] r;		    
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  return (i);
}

long arclength15 (long lcid)
{
  long j,k,n,ini,stop,li;
  double a0,a1,a2,l1,l2,dl,dlmax,dlmin,psi,lambda,blambda,bdlambda,dlambda,ddlambda,norf,norfp,norv,zero,ierr;
  double ss1,ss2,ss3,ss4,ss5;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  //  FILE *gr;
  //  char file[255];
  // double d;


  //  gr = fopen (Mp->auxfile,"a");
  
  n = Ndofm;                   
  ini = Mp->nlman.niilal;           
  zero = Mp->zero;          
  ierr = Mp->nlman.erral;           
  dl = Mp->nlman.dlal;              
  dlmax = Mp->nlman.dlmaxal;        
  dlmin = Mp->nlman.dlminal;        
  psi = Mp->nlman.psial;          
  
  r = new double [n];     ddr = new double [n];   u = new double [n];     v = new double [n];   
  f = new double [n];     fa = new double [n];    fi = new double [n];  
  
  //  Mm->alloc_backup (); 
  ra = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  
  arclopen_adapt (li,lambda,dlambda,dl);
  
  //  aux_mech_nonlin_print (gr,ra,lambda);
  // *******************************************************************
  //  backup of left hand side vector
  fprintf (stdout,"\n arc-length15: lambda %e  dl %e",lambda,dl);
  
  stiffness_matrix (lcid);
  for (j=0;j<n;j++)    f[j]=fp[j];

  Smat->solve_system (v,f);
  norv = displincr (v,n);
  norfp = ss (fp,fp,n);
    
  if (0 && !Mm->plast){
    for (j=0;j<n;j++)
      ra[j]=(lambda+dlambda)*v[j];
    
    //    aux_mech_nonlin_print (gr,ra,lambda);
  }
  else{  
    stop = 0;

    for (j=0;j<n;j++){      r[j]=ra[j];    }
    blambda=lambda;
    bdlambda=dlambda;
    //    Mm->backup_state_var ();
    
    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    internal_forces (lcid,fi);
    
    for (k=0;k<n;k++)
      f[k]=fa[k]-fi[k];
    
    norf=ss(f,f,n);
    norf=sqrt(norf);    if (Mespr==1)  fprintf (stdout,"\n %e %e norf %e",lambda,dl,norf);
    
    if (norf<ierr){
      lambda+=dlambda;
      Mm->updateipval ();
      stop = 1;
    }

    while (!stop){


      
      // ****************************
      //  inner iteration loop  ****   arclength15
      // ****************************
      for (j=0;j<10*ini;j++){
	
      //      aux_mech_nonlin_print (gr,ra,lambda);
      //  back substitution
	Smat->solve_system (u,f);
	
	for (k=0;k<n;k++){
	  f[k]=ddr[k];
	  ddr[k]+=u[k];
	}
	
	//  coefficient of quadratic equation
	//a2=norv+psi*psi*norfp;
	//a1=2.0*ss(ddr,v,n)+2.0*ddlambda*psi*psi*norfp;
	//a0=ss(ddr,ddr,n)+ddlambda*ddlambda*psi*psi*norfp-dl*dl;
	
	quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
	
	
	//  solution of quadratic equation
	/*
	if (fabs(a2)<zero){
	  if (fabs(a1)<zero){
	    if (fabs(a0)>zero)  fprintf (stderr,"\n\n nonsolvable constrained condition in function arclength");
	  else  fprintf (stderr,"\n\n infinite number of solution of constrained condition in function arclength");
	  }
	  else{
	    dlambda=(0.0-a0)/a1;
	  }
	}
	else{
	  d=a1*a1-4.0*a2*a0;
	  
	  if (d<0.0){
	    fprintf (stderr,"\n negative discriminant in function arclength");
	    break;
	  }
	  
	  
	  l1=(0.0-a1+sqrt(d))/2.0/a2;
	  l2=(0.0-a1-sqrt(d))/2.0/a2;
	  
	  if (l1>l2)  dlambda=l1;
	  else  dlambda=l2;
	  \\if (fabs(l1)<fabs(l2))  dlambda=l1;
	  \\else  dlambda=l2;
	}
	*/
	//  zacatek novinky   arclength15
        /*
	ss1=0.0;  ss2=0.0;
	ss3=0.0;  ss4=0.0;  ss5=0.0;
	for (k=0;k<n;k++){
	  ss1+=(ddr[k]+l1*v[k])*f[k];
	  ss2+=(ddr[k]+l2*v[k])*f[k];
	  ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
	  ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
	  ss5+=f[k]*f[k];
	}
	*/
	if (ss1/ss3/ss5>ss2/ss4/ss5)  dlambda=l1;
	else          dlambda=l2;
	//  konec novinky
	
        dlambda = 0;
	
	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	internal_forces (lcid,fi);
	
	for (k=0;k<n;k++)
	  f[k]=fa[k]-fi[k];
	
	norf=ss(f,f,n);   norf=sqrt(norf);
	
	if (Mespr==1){ 
          fprintf (stdout,"\n increment  %ld     inner loop  %ld    norf=%e",li,j,norf); 
//          fprintf (Out,"\n increment  %ld     inner loop  %ld    norf=%e",li,j,norf);
        }
	
	if (norf<0.10*ierr){
	  lambda+=ddlambda;
	  //	  aux_mech_nonlin_print (gr,ra,lambda);
	  Mm->updateipval ();
	  stop=1;  break;
	}
      }   // arclength15
      
      if (stop==0){
	//  modification of the arc length
	dl/=2.0;
	if (dl<dlmin){  dl=dlmin;  break; }
	
	if (Mespr==1){
	  fprintf (stdout,"\n arc length must be modified (dl decrease) dl=%e    norf=%e",dl,norf);
//	  fprintf (Out,"\n arc length must be modified (dl decrease) dl=%e    norf=%e",dl,norf);
	}
	//  restoring of left hand side vector
	for (j=0;j<n;j++){
	  ra[j]=r[j];
	}
	//  restoring of lambda parameter
	lambda=blambda;
	ddlambda=bdlambda;
	//  restoring of state variables
	//	Mm->restore_state_var ();

	//****************
	stop=1;
	//****************

      }
    }
  }
  
  // ------------------------------------
  //  finish of main iteration loop  ----  arclength15
  // ------------------------------------
  
  arclsave_adapt (li,lambda,0.0,dl);
  
  delete [] r;
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  //  fclose (gr); 
  
  return (li);
} //arclength15

void newton_raphson (long lcid)
  //  function solves system of nonlinear algebraic
  //  equations by Newton-Raphson method
  //  solved system does not contain time variable
  //
  //  16.8.2001
{
  long i,j,k,n,ni,ini;
  double lambda,blambda,dlambda,dlambdamin,zero,err,norf,norfa;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp;


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;

  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  print_init(-1, "wt");
  print_step(lcid, 0, 0.0, f);
  print_flush();
  for (i=0;i<ni;i++){
    
    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]=dlambda*fp[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    Smat->solve_system (dr,fb);
    
    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=ss(fb,fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,n);
    norf /= norfa;
    
//    fprintf (Out,"\n norf=%e",norf);

    if (norf<err){
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
      continue;
    }
    
    //  internal iteration loop
    for (j=0;j<ini;j++){
      
      //  solution of K(r).v=F
      Smat->solve_system (dr,fb);
      
      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,n);
      norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
//      fprintf (Out,"\n j=%ld     norf=%e",j,norf);
      
      if (norf<err)  break;
    }
    
    if (j==ini || norf>err){
      dlambda/=2.0;
      if (dlambda<dlambdamin)  dlambda=dlambdamin;

      if (Mespr==1){
	fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
//	fprintf (Out,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      }
      
      for (j=0;j<n;j++){
      	r[j]=rb[j];
      }
      lambda=blambda;
    }
    else{
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
    
  }
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  print_close();
}

void displ_control (long lcid)
  //  function solves system of nonlinear algebraic
  //  equations by Newton-Raphson method
  //  solved system does not contain time variable
  //  displacement contorl is assumed
  //
  //  2.11.2005
  //  TKo.
{
  long i,j,k,n,ni,ini;
  double lambda,blambda,dlambda,dlambdamin,zero,err,norf,norfa;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp, *fir2;


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;

  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  fir2 = new double[n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;
  Mp->lambda = lambda;
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  print_init(-1, "wt");
  print_step(lcid, 0, 0.0, f);
  print_flush();
  for (i=0;i<ni;i++){
    
    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
    Mp->lambda = lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    Mp->lambda += dlambda;
    internal_forces(lcid, fir2);

    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]= f[j] + dlambda*fp[j] - fir2[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);
    
    //  solution of K(r).v=F
    Smat->solve_system (dr,fb);
    
    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=ss(fb,fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,n);
    if (norfa != 0.0)
      norf /= norfa;
    
    if (norf<err){
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
      continue;
    }
    
    //  internal iteration loop
    for (j=0;j<ini;j++){
      
      //  solution of K(r).v=F
      Smat->solve_system (dr,fb);
      
      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,n);
      if (norfa != 0.0)
        norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
      
      if (norf<err)  break;
    }
    
    if (j==ini || norf>err)
    {
      if (dlambda == dlambdamin)
      {
        if (Mespr==1)  fprintf (stdout,"\n increment cannot be decreased dlambda=%e    norf=%e",dlambda,norf);
        break;
      }
      dlambda/=2.0;
      if (dlambda<dlambdamin)  dlambda=dlambdamin;
      if (Mespr==1)  fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      
      for (j=0;j<n;j++)
      	r[j]=rb[j];

      lambda=blambda;     
    }
    else
    {
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
  }
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  print_close();
}

void newton_raphson_restart (long lcid)
  //  function solves system of nonlinear algebraic
  //  equations by Newton-Raphson method
  //  solved system does not contain time variable
  //
  //  16.8.2001
{
  long i,j,k,n,ni,ini,nii;
  double lambda,blambda,dlambda,dlambdamin,zero,err,norf,norfa;
  double *r,*rb,*dr,*f,*fi,*fb,*fc,*fp;


  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.ninr;
  // number of initial increments
  nii = Mp->nlman.nienr;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilnr;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  err = Mp->nlman.errnr;
  //  increment size
  dlambda=Mp->nlman.incrnr;
  //  minimum increment size
  dlambdamin=Mp->nlman.minincrnr;

  rb = new double [n];
  f  = new double [n];
  fb = new double [n];
  fi = new double [n];
  dr = new double [n];
  memset (rb,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  r  = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  lambda=0.0;




  long ncompstr;
  double tmp;
  FILE *aux;

  // if restart from backup file is required
  if (Mp->nlman.hdbr)
  {
    aux = fopen (Mp->nlman.backupfname, "rt");
    if (aux == NULL)
    {
      fprintf(stderr, "\n\nError - unable to open backup file %s\n", Mp->nlman.backupfname);
      fprintf(stderr, "file %s, line %d\n", __FILE__, __LINE__);
      fprintf(stderr, "Program will be terminated\n");
      abort();
    }
    // searching required backup
    fprintf(stdout, "\n");
    for (k=0; k<Mp->nlman.hdbid; k++)
    {
      fprintf(stdout, "\r reading backup record %ld", k+1);
      fflush(stdout);
      mtsolver_restore(aux, r, fc, i, tmp, j, 39, 8);
    }
    fprintf(stdout, "\n backup time: %e\n", tmp);
    if (j != Ndofm)
    {
      fprintf(stderr, "\n\nError - incorrect number of DOFs is stored in backup file\n");
      fprintf(stderr, "file %s, line %d\n", __FILE__, __LINE__);
    }
/*
  Begining of Hinkley modification
*/
    for (i=0; i<Mm->tnip; i++)
    {
      if (Mm->ip[i].ncompeqother >= 8)
      {
        ncompstr = Mm->ip[i].ncompstr;
        for (j=0; j<ncompstr; j++)
          Mm->ip[i].stress[j] = Mm->ip[i].eqother[ncompstr+j];
      }
    }
  }

  Mp->strcomp = 0;
  internal_forces(lcid, fc);
  for (i=0; i < Ndofm; i++)
    fc[i] *= 1.0e+6;
  Mp->strcomp = 1;
/*
 End of Hinkley modification
*/


  // compute and print values initial values
  compute_req_val (lcid);
  print_init(-1, "wt");
  print_step(lcid, 0, 0, fc);
  print_flush();


  //  assembling of tangent stiffness matrix
  stiffness_matrix (lcid);
  // norm of load vector
  norfa=ss(fc,fc,n);
  //  internal iteration loop
  for (j=0;j<nii;j++)
  {
    //  computation of internal forces
    internal_forces (lcid,fi);
    //  vector of unbalanced forces
    for (k=0;k<n;k++)
      fb[k]=fc[k]-fi[k];
    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,n);
    norf /= norfa;
    
    if (norf<err)
    {
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, j+1, -0.0000001, fc);
      print_flush();
      break;
    }
    Smat->solve_system (dr,fb);
/*
    double tmp1, tmp2, tmp3, tmp4;
    long ncomp;
    tmp1=tmp2=tmp3=tmp4=0.0;
    fprintf(Out, "\n\n*Krok %ld\n", j);
    fprintf(Out,"\n\nn\tfc\tfi\tfc-fi\tr\tr+dr\tddr\n");
    for(k=0; k<n; k++)
    {
      tmp1+=fc[k]*fc[k];
      tmp2+=fi[k]*fi[k];
      tmp3+=r[k]*r[k];
      r[k]+=dr[k];
      tmp4+=r[k]*r[k];
      if (k%100==0)
        fprintf(Out, "%ld\t%e\t%e\t%e\t%e\t%e\t%e\n",k, tmp1, tmp2, tmp1-tmp2, tmp3, tmp4, tmp3-tmp4);
    }
    fprintf(Out, "%ld\t%e\t%e\t%e\t%e\t%e\t%e\n",k, tmp1, tmp2, tmp1-tmp2, tmp3, tmp4, tmp3-tmp4);
    fprintf(Out,"\n\nIpp\t omega\n");
   
    for (k=0; k < Mm->tnip; k++)
    {
      if (Mm->ip[k].ncompother > 8)
      {
        ncomp = 2*Mm->ip[k].ncompstr;
        fprintf(Out, "%ld\t%e\t%e\n", k, Mm->ip[k].other[ncomp],Mm->ip[k].other[ncomp+1]);
      }
    }
    fflush(Out);*/
    for(k=0; k<n; k++)
      r[k]+=dr[k];
/*    swap_other();
    compute_req_val (lcid);
    print_step(lcid, j+1, j+1, fc);
    print_flush();
    swap_other();*/
    fprintf (stdout,"\n Equilibrium interation loop j %ld     norf=%e", j, norf);
  }
  if ((nii > 0) && (norf > err))
  {
    fprintf(stdout, "\n Initial equilibrium could not be reached\n");
    fprintf(stdout, " Program will be terminated\n");
    print_close();
    return;
  }
  else
    fprintf(stdout, "\n\n Starting incremental loading . . .\n");


  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=0;i<ni;i++){
    
    //  backup of left hand side vector
    for (j=0;j<n;j++){
      rb[j]=r[j];
    }
    //  backup of reached lambda parameter
    blambda=lambda;
   
    fprintf (stdout,"\n increment %ld   lambda %e,    dlambda %e",i,lambda,dlambda);
    
    //  vector of maximum load and vector of load increment
    for (j=0;j<n;j++){
      f[j]=fc[j]+lambda*fp[j];
      fb[j]=dlambda*fp[j];
    }
    
    //  assembling of tangent stiffness matrix
    if ((Mp->stmat==tangent_stiff) || (i == 0))
      stiffness_matrix (lcid);

    //  solution of K(r).v=F

    Smat->solve_system (dr,fb);
    
    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    //  computation of internal forces
    internal_forces (lcid,fi);
    //  vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j]=f[j]+dlambda*fp[j];
    }
    norfa=ss(fb,fb,n);
    for (j=0;j<n;j++){
      fb[j] -= fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,n);
    norf /= norfa;
    
    
    
//    fprintf (Out,"\n norf=%e",norf);

    if (norf<err){
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
      continue;
    }
    
    //  internal iteration loop
    for (j=0;j<ini;j++){
      
      //  solution of K(r).v=F
      Smat->solve_system (dr,fb);
      
      for (k=0;k<n;k++){
	r[k]+=dr[k];
      }
      
      //  computation of internal forces
      internal_forces (lcid,fi);
      
      //  vector of unbalanced forces
      for (k=0;k<n;k++){
	fb[k]=f[k]+dlambda*fp[k]-fi[k];
      }
      
      //  norm of vector of unbalanced forces
      norf=ss(fb,fb,n);
      norf /= norfa;
      
      fprintf (stdout,"\n increment %ld   inner loop j %ld     norf=%e  dlambda %e",i,j,norf,dlambda);
//      fprintf (Out,"\n j=%ld     norf=%e",j,norf);
      
      if (norf<err)  break;
    }
    
    if (j==ini || norf>err){
      dlambda/=2.0;
      if (dlambda<dlambdamin)  dlambda=dlambdamin;

      if (Mespr==1){
	fprintf (stdout,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
//	fprintf (Out,"\n increment must be modified (dlambda decrease) dlambda=%e    norf=%e",dlambda,norf);
      }
      
      for (j=0;j<n;j++){
      	r[j]=rb[j];
      }
      lambda=blambda;
    }
    else{
      lambda+=dlambda;
      Mm->updateipval ();
      compute_req_val (lcid);
      print_step(lcid, i+1, lambda, f);
      print_flush();
    }
    
  }
  
  delete [] dr;  delete [] fi;  delete [] fb;  delete [] f;
  print_close();
}

void seldofinit ()
{
  long i,j,k,l,ndofn;
  double x1,x2,y1,y2,z1,z2,length;
  
  switch (Mp->nlman.displnorm){
  case alldofs:{  break; }
  case seldofs:
  case seldofscoord:{
    for (i=0;i<Mp->nlman.nsdofal;i++){
      j=Mp->nlman.seldofal[i];
      Mp->nlman.seldofal[i]=Mt->give_dof (Mp->nlman.selnodal[i],j)-1;
    }
    break;
  }
  case selnodes:{
    Mp->nlman.nsdofal=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      Mp->nlman.nsdofal+=Mt->give_ndofn (Mp->nlman.selnodal[i]);
    }
    Mp->nlman.seldofal = new long [Mp->nlman.nsdofal];
    k=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      ndofn=Mt->give_ndofn (Mp->nlman.selnodal[i]);
      for (j=0;j<ndofn;j++){
	l=Mt->give_dof (Mp->nlman.selnodal[i],j);
	if (l>0){
	  Mp->nlman.seldofal[k]=l-1;
	  k++;
	}
      }
    }
    Mp->nlman.nsdofal=k;
    break;
  }
  case nodesdistincr:{
    Mp->nlman.nsdofal=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      Mp->nlman.nsdofal+=Mt->give_ndofn (Mp->nlman.selnodal[i]);
    }
    Mp->nlman.seldofal = new long [Mp->nlman.nsdofal];
    k=0;
    for (i=0;i<Mp->nlman.nsnal;i++){
      ndofn=Mt->give_ndofn (Mp->nlman.selnodal[i]);
      for (j=0;j<ndofn;j++){
	Mp->nlman.seldofal[k]=Mt->give_dof (Mp->nlman.selnodal[i],j);
	k++;
      }
    }
    x1=Gtm->gnodes[Mp->nlman.selnodal[0]].x;    x2=Gtm->gnodes[Mp->nlman.selnodal[1]].x;
    y1=Gtm->gnodes[Mp->nlman.selnodal[0]].y;    y2=Gtm->gnodes[Mp->nlman.selnodal[1]].y;
    z1=Gtm->gnodes[Mp->nlman.selnodal[0]].z;    z2=Gtm->gnodes[Mp->nlman.selnodal[1]].z;
    length=sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)+(z2-z1)*(z2-z1));
    Mp->nlman.nxal=(x2-x1)/length;  Mp->nlman.nyal=(y2-y1)/length;  Mp->nlman.nzal=(z2-z1)/length;
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown displacement norm is required in function seldofinit (%s, line %d)",__FILE__,__LINE__);
  }
  }
}

double displincr (double *displincr,long n)
{
  long i;
  double norm,u1,u2,v1,v2,w1,w2;
  
  switch (Mp->nlman.displnorm){
  case alldofs:{
    norm = ss (displincr,displincr,n);
    break;
  }
  case seldofs:
  case seldofscoord:
  case selnodes:{
    norm=0.0;
    for (i=0;i<Mp->nlman.nsdofal;i++){
      norm+=displincr[Mp->nlman.seldofal[i]]*displincr[Mp->nlman.seldofal[i]];
    }
    break;
  }
  case nodesdistincr:{
    if (Mp->nlman.probdimal==2){
      u1=0.0;  u2=0.0;  v1=0.0;  v2=0.0;
      if (Mp->nlman.seldofal[0]>0)  u1=displincr[Mp->nlman.seldofal[0]-1];
      if (Mp->nlman.seldofal[1]>0)  v1=displincr[Mp->nlman.seldofal[1]-1];
      if (Mp->nlman.seldofal[2]>0)  u2=displincr[Mp->nlman.seldofal[2]-1];
      if (Mp->nlman.seldofal[3]>0)  v2=displincr[Mp->nlman.seldofal[3]-1];
      
      norm=(u2-u1)*Mp->nlman.nxal + (v2-v1)*Mp->nlman.nyal;
    }
    if (Mp->nlman.probdimal==3){
      u1=0.0;  u2=0.0;  v1=0.0;  v2=0.0;  w1=0.0;  w2=0.0;
      if (Mp->nlman.seldofal[0]>0)  u1=displincr[Mp->nlman.seldofal[0]-1];
      if (Mp->nlman.seldofal[1]>0)  v1=displincr[Mp->nlman.seldofal[1]-1];
      if (Mp->nlman.seldofal[2]>0)  w1=displincr[Mp->nlman.seldofal[2]-1];
      if (Mp->nlman.seldofal[3]>0)  u2=displincr[Mp->nlman.seldofal[3]-1];
      if (Mp->nlman.seldofal[4]>0)  v2=displincr[Mp->nlman.seldofal[4]-1];
      if (Mp->nlman.seldofal[5]>0)  w2=displincr[Mp->nlman.seldofal[5]-1];
      
      norm=(u2-u1)*Mp->nlman.nxal + (v2-v1)*Mp->nlman.nyal + (w2-w1)*Mp->nlman.nzal;
    }
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown norm of displacement increment is required in function");
    fprintf (stderr,"\n displincr (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
  norm=sqrt (norm);
  return norm;
}


void quadeqcoeff (double *ddr,double *v,long n,double ddlambda,double psi,double norfp,double dl,
		  double &a0,double &a1,double &a2)
{
  long i;
  
  switch (Mp->nlman.displnorm){
  case alldofs:{
    a2=ss(v,v,n)+psi*psi*norfp;
    a1=2.0*ss(ddr,v,n)+2.0*ddlambda*psi*psi*norfp;
    a0=ss(ddr,ddr,n)+ddlambda*ddlambda*psi*psi*norfp-dl*dl;
    break;
  }
  case seldofs:
  case seldofscoord:
  case selnodes:{
    a0=0.0;  a1=0.0;  a2=0.0;
    for (i=0;i<Mp->nlman.nsdofal;i++){
      a0+=ddr[Mp->nlman.seldofal[i]]*ddr[Mp->nlman.seldofal[i]];
      a1+=ddr[Mp->nlman.seldofal[i]]*v[Mp->nlman.seldofal[i]]*2.0;
      a2+=v[Mp->nlman.seldofal[i]]*v[Mp->nlman.seldofal[i]];
    }
    a0-=dl*dl;
    break;
  }
  case nodesdistincr:{
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown norm of displacement increment is required in function");
    fprintf (stderr,"\n displincr (%s, line %d).\n",__FILE__,__LINE__);
  }
  }

}

void arclsave (long fi,long n,double blambda,double dl,double *r,double *fp)
{
  long i,j;
  char *file;
  FILE *aux;
  file = new char[strlen(Mp->path)+11+1];
  sprintf (file,"%sarcl.backup",Mp->path);
  aux = fopen (file,"w");

  //aux = fopen ("arcl.backup","w");
  
  //  attained number of steps
  fprintf (aux,"%ld\n",fi);
  //  number of rows
  fprintf (aux,"%ld\n",n);
  //  attained coefficient of proportionality
  fprintf (aux,"%e\n",blambda);
  //  arc-length
  fprintf (aux,"%e\n",dl);
  //  number of selected dofs
  fprintf (aux,"%ld\n",Mp->nlman.nsdofal);
  //  selected dofs
  for (i=0;i<Mp->nlman.nsdofal;i++){
    fprintf (aux,"%ld\n",Mp->nlman.seldofal[i]);
  }
  //  left hand side vector
  for (i=0;i<n;i++){
    fprintf (aux,"%e\n",r[i]);
  }
  //  proportionality vector
  for (i=0;i<n;i++){
    fprintf (aux,"%e\n",fp[i]);
  }
  //  number of integration points
  fprintf (aux,"%ld\n",Mm->tnip);
  //  integration points
  for (i=0;i<Mm->tnip;i++){
    fprintf (aux,"%ld %ld\n",Mm->ip[i].ncompstr,Mm->ip[i].ncompother);
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fprintf (aux,"%e\n",Mm->ip[i].strain[j]);
    }
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fprintf (aux,"%e\n",Mm->ip[i].stress[j]);
    }
    for (j=0;j<Mm->ip[i].ncompother;j++){
      fprintf (aux,"%e\n",Mm->ip[i].eqother[j]);
    }
  }
  
  fclose (aux);
}

void arclopen (long &fi,long &n,double &blambda,double &dl,double *r,double *fp)
{
  long i,j;
  char *file;
  FILE *aux;
  //  aux = fopen ("arcl.backup","r");
  // termit
  file = new char[strlen(Mp->path)+11+1];
  sprintf (file,"%sarcl.backup",Mp->path);
  aux = fopen (file,"r");
  
  //  attained number of steps
  fscanf (aux,"%ld",&fi);
  //  number of rows
  fscanf (aux,"%ld",&n);
  //  attained coefficient of proportionality
  fscanf (aux,"%le",&blambda);
  //  arc-length
  fscanf (aux,"%le",&dl);
  //  number of selected dofs
  fscanf (aux,"%ld",&Mp->nlman.nsdofal);
  //  selected dofs
  for (i=0;i<Mp->nlman.nsdofal;i++){
    fscanf (aux,"%ld",&Mp->nlman.seldofal[i]);
  }
  //  left hand side vector
  for (i=0;i<n;i++){
    fscanf (aux,"%le",&r[i]);
  }
  //  proportionality vector
  for (i=0;i<n;i++){
    fscanf (aux,"%le",&fp[i]);
  }
  //  number of integration points
  fscanf (aux,"%ld",&Mm->tnip);
  //  integration points
  for (i=0;i<Mm->tnip;i++){
    fscanf (aux,"%ld %ld",&Mm->ip[i].ncompstr,&Mm->ip[i].ncompother);
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fscanf (aux,"%le",&Mm->ip[i].strain[j]);
    }
    for (j=0;j<Mm->ip[i].ncompstr;j++){
      fscanf (aux,"%le",&Mm->ip[i].stress[j]);
    }
    for (j=0;j<Mm->ip[i].ncompother;j++){
      fscanf (aux,"%le",&Mm->ip[i].eqother[j]);
    }
  }
  
  fclose (aux);
}



/**

   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void arclsave_adapt (long fi,double lambda,double dlambda,double dl)
{
  char *file;
  FILE *aux;
  
  file = new char[strlen(Mp->path)+19+1];
  //  sprintf (file,"%sarcl.backup.ada.%03ld",Mp->path,Mp->nial);
  sprintf (file,"%sarcl.backup.ada",Mp->path);
  aux = fopen (file,"w");
  
  //  attained number of steps
  fprintf (aux,"%ld\n",fi);
  
  //  attained coefficient of proportionality
  fprintf (aux,"%.16e\n",lambda);
  
  //  
  fprintf (aux,"%.16e\n",dlambda);
  
  //  arc-length
  fprintf (aux,"%.16e\n",dl);
  
  fclose (aux);
}
/**

   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void arclopen_adapt (long &fi,double &lambda,double &dlambda,double &dl)
{
  char *file;
  FILE *aux;
  
  file = new char[strlen(Mp->path)+19+1];
  //  sprintf (file,"%sarcl.backup.ada.%03ld",Mp->path,Mp->nial);
  sprintf (file,"%sarcl.backup.ada",Mp->path);
  aux = fopen (file,"r");
  
  //  attained number of steps
  fscanf (aux,"%ld",&fi);
  
  //  attained coefficient of proportionality
  fscanf (aux,"%le",&lambda);
  
  //  
  fscanf (aux,"%le",&dlambda);
  
  //  arc-length
  fscanf (aux,"%le",&dl);

  fclose (aux);
}

/**

   created  1.12.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void newmeshgen (int w,long i)
{
  char procname[255];
  
  if (Mespr==1)  fprintf (stdout,"\n\n ***  T3D generator - start  ***\n");
  
  if (Mespr) sprintf (procname,"/home/dr/Bin/T3d -i %sXmesht3d.in -o %smesh.out.%0*ld -m %smesh.bgm.%0*ld -d 100 -p 8 -r 1 -k %ld"
	                      ,Mp->path,Mp->path,w,i,Mp->path,w,i-1,Mt->give_degree(0));
  else       sprintf (procname,"/home/dr/Bin/T3d -i %sXmesht3d.in -o %smesh.out.%0*ld -m %smesh.bgm.%0*ld -d 100 -p 8 -r 1 -k %ld  > /dev/null 2>&1"
	                      ,Mp->path,Mp->path,w,i,Mp->path,w,i-1,Mt->give_degree(0));
  
  if (Mespr==1)  fprintf (stdout,"\n\n ***  T3D generator - end  *****\n");


  system (procname);
  
  
  sprintf (procname,"sed  -e s/out.i/out.%0*ld/g  -e s/dat.i/dat.%d/g  %sprep.pre > %sprep.%0*ld",w,i,0,Mp->path,Mp->path,w,i);
  system (procname);
  
  if (Mespr)
    sprintf (procname,"./mechprep %sprep.%0*ld %ssifel.in.%0*ld",Mp->path,w,i,Mp->path,w,i);
  else
    sprintf (procname,"./mechprep %sprep.%0*ld %ssifel.in.%0*ld  > /dev/null 2>&1",Mp->path,w,i,Mp->path,w,i);

  system (procname);
  

  sprintf (procname,"rm -f %sprep.%0*ld",Mp->path,w,i);
  system (procname);
}

/*
      FILE *ff;
      ff = fopen ("problemnl/aux.txt","a");      if (ff==NULL) fprintf (stderr,"\n ff file has not been opened.\n\n");
      for (long g=0;g<Ndofn;g++)
	fprintf (ff,"%40.20lf\n",Lsrs->lhs[i]);
      fclose (ff);
      
      


*/

/**
   function solves system of nonlinear algebraic equation
   by arc-length method (continuation method)
   only one right hand side vector is supported with respect
   to nonlinearity and absence of superposition

   d stands for delta
   dd stands for capital delta
   
   fc - nonproportional %vector
   fp - proportional %vector
   n - order of the system
   
   @param lcid - load case id
   
   23.12.2004
*/
long arclengthadapt (long lcid,long adaptcontrol)
{
  long i,j,k,n,ni,ini,stop,modif,li,newmesh, numr;
  double a0,a1,a2,l1,l2,dl,dlmax,dlmin,psi,lambda,blambda,dlambda,ddlambda,norf,norfp,norv,norfa,zero,ierr;
  double lambdao,ss1,ss2,ss3,ss4,ss5;
  double *r,*ra,*ddr,*u,*v,*f,*fa,*fp,*fc,*fi;
  
  
  
  //  number of rows of the matrix
  n = Ndofm;
  //  maximum number of increments
  ni = Mp->nlman.nial;
  //  maximum number of iterations in one increment
  ini = Mp->nlman.niilal;
  //  computer zero
  zero = Mp->zero;
  //  required error in inner loop
  ierr = Mp->nlman.erral;
  //  length of arc
  dl = Mp->nlman.dlal;
  //  maximum length of arc
  dlmax = Mp->nlman.dlmaxal;
  //  minimum length of arc
  dlmin = Mp->nlman.dlminal;
  //  displacement-loading driving switch
  psi = Mp->nlman.psial;
  
  dlambda=0.0;
  
  //  allocation of auxiliary arrays
  r = new double [n];
  ddr = new double [n];
  u = new double [n];
  v = new double [n];
  f = new double [n];
  fa = new double [n];
  fi = new double [n];
  
  
  //  initialization phase
  ra = Lsrs->give_lhs (lcid);
  fp = Lsrs->give_rhs (lcid*2);
  fc = Lsrs->give_rhs (lcid*2+1);
  
  if (Mp->nlman.hdbackupal == 2){            // termit
    arclopen (li,n,lambda,dl,ra,fp);
  }
  else if (adaptcontrol >= 2){
    arclopen_adapt (li,lambda,blambda,dl);
  }
  else{
    lambda=0.0;
    lambdao=0.0;
    li=0;
  }
  
  //  norm of proportionality vector
  norfp = ss(fp,fp,n);
  modif=0;


  // assembling reached load vector
  for (j=0;j<n;j++){
    fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    f[j]=fp[j];
  }
  if (li)
    print_init(-1, "at");
  else
  {
    print_init(-1, "wt");
    print_step(lcid, li, lambda, fa);
  }
  
  
  //  elastic trial response of structure
  Mp->phase=1;
  //  stiffness matrix based on elastic models
  stiffness_matrix (lcid);
  //  solution of K.r=f
  Smat->solve_system (ra,f);
  
  Mm->computestresses (0);

  //  switch to the special material models
  Mp->phase=2;
  
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  for (i=li;i<ni;i++){
    stop=1; // termit
    //  backup of left hand side vector
    copyv (ra, r, n);
    //  backup of reached lambda parameter
    blambda=lambda;

    
    //  assembling of tangent stiffness matrix
    if ((Mp->stmat==tangent_stiff) || (i == li))
      stiffness_matrix (lcid);
    
    //  backup of the fp, in ldl_sky the right hand side will be destroyed
    copyv(fp, f, n);

    //  solution of K(r).v=F
    Smat->solve_system (v,f);

    //  generalized norm of displacement increments
    norv = displincr (v,n);

    //  compute new dlambda increment
    dlambda = dl/sqrt(norv+psi*psi*norfp);
    
    for (j=0;j<n;j++){
      ddr[j]=dlambda*v[j];
      ra[j]+=ddr[j];
      fa[j]=fc[j]+(lambda+dlambda)*fp[j];
    }
    ddlambda=dlambda;
    
    //  computation of internal forces
    //Mm->computestresses (0);
    internal_forces (lcid,fi);
    subv(fa, fi, f, n);    
    norf=sizev(f,n);
    norfa = sizev(fa, n);
    norf /= norfa;

    
    //if (Mespr==1)  fprintf (stdout,"\n ddlambda %e    dlambda %e   norf %e  ierr %e",ddlambda,dlambda,norf,ierr);
    if (Mespr==1)  fprintf (stdout,"\n increment %ld     norv %e      norf %e",i,norv,norf);
    
    if (norf<ierr){
      // ******************************************
      //  no inner iteration loop is required  ***
      // ******************************************
      modif++;

      if (modif>1){
	//  arc length modification
	dl*=2.0;
	if (dl>dlmax)  
          dl=dlmax;
	modif=0;
	if (Mespr==1){
	  fprintf (stdout,"\n arc-length must be modified (dl increase) dl=%e",dl);
//	  fprintf (Out,"\n arc-length must be modified (dl increase) dl=%e",dl);
	}
      }
      lambda+=dlambda;
      Mm->updateipval ();
      print_step(lcid, i+1, lambda, fa);
      print_flush();
    }
    else{
      // ****************************
      //  inner iteration loop  ****
      // ****************************
      stop=0;
      for (j=0;j<ini;j++){
	
	if (Mp->stmat==tangent_stiff)
	  stiffness_matrix (lcid);
	
	//  back substitution
	Smat->solve_system (u,f);
	
        copyv(ddr, f, n);
        addv(ddr, u, n);
	//  coefficient of quadratic equation
	quadeqcoeff (ddr,v,n,ddlambda,psi,norfp,dl,a0,a1,a2);
	/*
	  fprintf (Out,"\n\n\n Kontrola kvadraticke rovnice");
	  fprintf (Out,"\n norfp     %15.10le",norfp);
	  fprintf (Out,"\n norv      %15.10le",norv);
	  fprintf (Out,"\n (ddr,v)   %15.10le",ss(ddr,v,n));
	  fprintf (Out,"\n (ddr,ddr) %15.10le",ss(ddr,ddr,n));
	  fprintf (Out,"\n dl        %15.10le",dl);
	  fprintf (Out,"\n ddlambda  %15.10le",ddlambda);
	  fprintf (Out,"\n psi       %15.10le",psi);
	  fprintf (Out,"\n a2        %15.10le",a2);
	  fprintf (Out,"\n a1        %15.10le",a1);
	  fprintf (Out,"\n a0        %15.10le",a0);
	  fprintf (Out,"\n discrim   %15.10le",a1*a1-4.0*a2*a0);
	  fprintf (Out,"\n\n");
	*/
	
	//  solution of quadratic equation
        numr = solv_polynom_2(a2, a1, a0, l1, l2);
        switch (numr)
	{
	  case -1:
            fprintf (stderr,"\n\n infinite number of solution of constrained condition in function arclength");
            break;
	  case 0:
            fprintf (stderr,"\n\n nonsolvable constrained condition in function arclength");
            break;
    	  case 1:
            dlambda = l1;
            break;
	  default:
            break;
	}
	ss1=0.0;  ss2=0.0;
	ss3=0.0;  ss4=0.0;  ss5=0.0;
	for (k=0;k<n;k++){
	  ss1+=(ddr[k]+l1*v[k])*f[k];
	  ss2+=(ddr[k]+l2*v[k])*f[k];
	  ss3+=(ddr[k]+l1*v[k])*(ddr[k]+l1*v[k]);
	  ss4+=(ddr[k]+l2*v[k])*(ddr[k]+l2*v[k]);
	  ss5+=f[k]*f[k];
	}
	
	/*
	if (ss1/ss3/ss5>ss2/ss4/ss5)  
          dlambda=l1;
	else                          
          dlambda=l2;
	*/
	if (fabs(l1)>fabs(l2))  dlambda=l2;
	else dlambda=l1;
	
	
	//fprintf (stdout,"\n increment  %ld     inner loop  %ld   x1=%e x2=%e   dlambda %e",i,j,l1,l2,dlambda);
	//fprintf (stdout,"\n increment  %ld     inner loop  %ld   x1=%e x2=%e   ddlambda %e",i,j,l1,l2,ddlambda);

	for (k=0;k<n;k++){
	  ddr[k]+=dlambda*v[k];
	  ra[k]+=u[k]+dlambda*v[k];
	  fa[k]+=dlambda*fp[k];
	}
	ddlambda+=dlambda;
	
	//fprintf (stdout,"   ddlambda %e",ddlambda);
	
	//fprintf (Out,"\n ddlambda %e     dlambda %e",ddlambda,dlambda);
	//  computation of internal forces
	internal_forces (lcid,fi);
        subv(fa, fi, f, n);	
	norf=sizev(f,n);
	norfa = sizev(fa, n);
	norf /= norfa;

	if (Mespr==1){
	  fprintf (stdout,"\n    norf=%e  ierr=%e",norf,ierr);
	  //fprintf (Out,"\n increment  %ld     inner loop  %ld    norf=%e",i,j,norf);
	}
	if (norf<ierr){
	  lambda+=ddlambda;
	  Mm->updateipval ();
	  stop=1; 
          print_step(lcid, i+1, lambda, fa);
          print_flush();
	  break;
	}
      }
      modif=0;
      if (stop==0){
	//  modification of the arc length
	dl/=2.0;
	if (dl<dlmin){
          dl=dlmin;  
          break; 
        }
	if (Mespr==1){
	  fprintf (stdout,"\n arc length must be modified (dl decrease) dl=%e",dl);
	  //fprintf (Out,"\n arc length must be modified (dl decrease) dl=%e",dl);
	}
	//  restoring of left hand side vector
        copyv(r, ra, n);
	//  restoring of lambda parameter
	lambda=blambda;
      }
    }
    
    fprintf (stdout,"\n increment %ld    total lambda %e",i,lambda);

    if (stop==0)
      continue;
    
    if (Mp->adaptflag & 16 && i%10 && Mm->plast)
      continue;
    
    newmesh = 0;
    if (Mp->adaptivity && Mp->nlman.nial > (i+1) && !( (Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))){
      
      // print of fxlambda second part is in adaptivity.cpp
      //FILE *ff;
      //sprintf (file,"%sfxlambda.dat",Mp->path);
      //ff = fopen (file,"a");
      //if (ff==NULL) fprintf (stderr,"\n ff file has not been opened.\n\n");
      //fprintf (ff,"  %20.10f",lambda);
      //fclose (ff);
      
      long width = 1 + (long)log10((double)Mp->nlman.nial);
      newmesh = Ada->run (1,width,i);
    }
    
    if (newmesh){
      i++;             // toto se da odstranit
      break;
    }
    // timret
    //print_output(lcid);
  }
  
  // ------------------------------------
  //  finish of main iteration loop  ----
  // ------------------------------------
  
  if (Mp->nlman.hdbackupal==1)
    arclsave (i,n,lambda,dl,ra,fp);

  // termit
  if (Mp->adaptivity)
    if ((Mp->adaptflag & 16) && (Mp->nlman.nial==100 || Mp->nlman.nial==1))
      Ada->run (0,0,0);  // only for testing
    else
      if (i < ni)
	arclsave_adapt (i,lambda,0,dl);
  
  //if (Mp->adaptivity)   arclsave_adapt (i,blambda,ddlambda,dl);
  // timret
  print_close();
  delete [] r;		    
  delete [] fi;  
  delete [] fa;  
  delete [] f;
  delete [] v;  
  delete [] u;  
  delete [] ddr;
  
  return (i);
}
